@@ -7,7 +7,7 @@
  *
  * http://glpi-project.org
  *
- * @copyright 2015-2023 Teclib' and contributors.
+ * @copyright 2015-2025 Teclib' and contributors.
  * @copyright 2003-2014 by the INDEPNET Development Team.
  * @licence   https://www.gnu.org/licenses/gpl-3.0.html
  *
@@ -32,124 +32,146 @@
  *
  * ---------------------------------------------------------------------
  */
-
-$SECURITY_STRATEGY = 'no_check'; // specific checks done later to allow anonymous access to embed dashboards
-
-include('../inc/includes.php');
-
+use Glpi\Dashboard\Dashboard;
 use Glpi\Dashboard\Grid;
+use Glpi\Debug\Profiler;
+use Glpi\Error\ErrorHandler;
+use Glpi\Exception\Http\AccessDeniedHttpException;
+
+use function Safe\json_decode;
+use function Safe\json_encode;
 
 if (!isset($_REQUEST["action"])) {
-    exit;
+    return;
 }
 
 // Parse stringified JSON payload (Used to preserve integers)
-$request_data = array_merge($_REQUEST, json_decode($_UREQUEST['data'] ?? '{}', true));
+$request_data = array_merge($_REQUEST, json_decode($_REQUEST['data'] ?? '{}', true));
 unset($request_data['data']);
 
+// Session check is disabled for this script (see `\Glpi\Http\Firewall::computeStrategyForCoreLegacyScript()`)
+// to be able to adapt the checks depending on the request.
 $embed = false;
 if (
     in_array($_REQUEST['action'], ['get_dashboard_items', 'get_card', 'get_cards'])
     && array_key_exists('embed', $request_data)
-    && (bool)$request_data['embed']
+    && (bool) $request_data['embed']
 ) {
     if (Grid::checkToken($request_data) === false) {
-        http_response_code(403);
-        exit;
+        throw new AccessDeniedHttpException();
     }
     $embed = true;
 } else {
     Session::checkLoginUser();
 }
 
-$dashboard = new Glpi\Dashboard\Dashboard($_REQUEST['dashboard'] ?? "");
+$dashboard = new Dashboard($_REQUEST['dashboard'] ?? "");
 
 switch ($_POST['action'] ?? null) {
     case 'save_new_dashboard':
+        header("Content-Type: application/json; charset=UTF-8");
+
         if (!Session::haveRight('dashboard', CREATE)) {
-            http_response_code(403);
-            exit();
+            throw new AccessDeniedHttpException();
         }
 
-        echo $dashboard->saveNew(
+        $key = $dashboard->saveNew(
             $_POST['title']   ?? "",
             $_POST['context'] ?? ""
         );
-        exit;
+        echo json_encode($key);
+        return;
 
     case 'save_items':
         if (!$dashboard->canUpdateCurrent()) {
-            http_response_code(403);
-            exit();
+            throw new AccessDeniedHttpException();
         }
 
         $dashboard->saveitems($_POST['items'] ?? []);
         $dashboard->saveTitle($_POST['title'] ?? "");
-        exit;
+        return;
 
     case 'save_rights':
         if (!$dashboard->canUpdateCurrent()) {
-            http_response_code(403);
-            exit();
+            throw new AccessDeniedHttpException();
         }
 
         $dashboard->setPrivate($_POST['is_private'] != '0');
         $dashboard->saveRights($_POST['rights'] ?? []);
-        exit;
+        return;
 
     case 'save_filter_data':
         if (!$dashboard->canViewCurrent()) {
-            http_response_code(403);
-            exit();
+            throw new AccessDeniedHttpException();
         }
 
         $dashboard->saveFilter($_POST['filters'] ?? []);
-        exit;
+        return;
 
     case 'delete_dashboard':
+        header("Content-Type: application/json; charset=UTF-8");
+
         if (!$dashboard->canDeleteCurrent()) {
-            http_response_code(403);
-            exit();
+            throw new AccessDeniedHttpException();
         }
 
-        echo $dashboard->delete(['key' => $_POST['dashboard']]);
-        exit;
+        $success = $dashboard->delete(['key' => $_POST['dashboard']]);
+        echo json_encode($success);
+        return;
 
     case 'set_last_dashboard':
         $grid = new Grid($_POST['dashboard'] ?? "");
         $grid->setLastDashboard($_POST['page'], $_POST['dashboard']);
-        exit;
+        return;
 
     case 'clone_dashboard':
+        header("Content-Type: application/json; charset=UTF-8");
+
         if (!Session::haveRight('dashboard', CREATE) || !$dashboard->canViewCurrent()) {
-            http_response_code(403);
-            exit();
+            throw new AccessDeniedHttpException();
         }
 
         $new_dashboard = $dashboard->cloneCurrent();
         echo json_encode($new_dashboard);
-        exit;
+        return;
+
+    case 'disable_placeholders':
+        if (!Session::haveRight(Config::$rightname, UPDATE)) {
+            throw new AccessDeniedHttpException();
+        }
+        Config::setConfigurationValues('core', ['is_demo_dashboards' => 0]);
+        return;
 }
 
 switch ($_GET['action'] ?? null) {
     case 'get_filter_data':
+        header("Content-Type: application/json; charset=UTF-8");
+
         if (!$dashboard->canViewCurrent()) {
-            http_response_code(403);
-            exit();
+            throw new AccessDeniedHttpException();
         }
 
-        echo $dashboard->getFilter();
-        exit;
+        /**
+         * `Dashboard::getFilter()` already returns a JSON encoded string.
+         *
+         * @psalm-taint-escape has_quotes
+         * @psalm-taint-escape html
+         */
+        $filter = $dashboard->getFilter();
+
+        echo $filter;
+        return;
 }
 
+Profiler::getInstance()->start('Grid::construct');
 $grid = new Grid($_REQUEST['dashboard'] ?? "");
+Profiler::getInstance()->stop('Grid::construct');
 
 header("Content-Type: text/html; charset=UTF-8");
 switch ($_REQUEST['action']) {
     case 'add_new':
         if (!Session::haveRight('dashboard', CREATE)) {
-            http_response_code(403);
-            exit();
+            throw new AccessDeniedHttpException();
         }
 
         $grid->displayAddDashboardForm();
@@ -158,8 +180,7 @@ switch ($_REQUEST['action']) {
     case 'edit_rights':
         // FIXME This endpoint does not seems to be used.
         if (!Session::haveRight('dashboard', UPDATE)) {
-            http_response_code(403);
-            exit();
+            throw new AccessDeniedHttpException();
         }
 
         $grid->displayEditRightsForm();
@@ -168,8 +189,7 @@ switch ($_REQUEST['action']) {
     case 'display_edit_widget':
     case 'display_add_widget':
         if (!$dashboard->canUpdateCurrent()) {
-            http_response_code(403);
-            exit();
+            throw new AccessDeniedHttpException();
         }
 
         $grid->displayWidgetForm($_REQUEST);
@@ -177,8 +197,7 @@ switch ($_REQUEST['action']) {
 
     case 'display_embed_form':
         if (!Session::haveRight('dashboard', UPDATE)) {
-            http_response_code(403);
-            exit();
+            throw new AccessDeniedHttpException();
         }
 
         $grid->displayEmbedForm();
@@ -186,59 +205,59 @@ switch ($_REQUEST['action']) {
 
     case 'get_card':
         if (!$dashboard->canViewCurrent() && !$embed) {
-            http_response_code(403);
-            exit();
+            throw new AccessDeniedHttpException();
         }
 
         Session::writeClose();
+        Profiler::getInstance()->start('Get card HTML');
+        if ($embed) {
+            $grid->initEmbedSession($_REQUEST);
+        }
         echo $grid->getCardHtml($_REQUEST['card_id'], $_REQUEST);
+        Profiler::getInstance()->stop('Get card HTML');
         break;
 
     case 'get_cards':
+        header("Content-Type: application/json; charset=UTF-8");
+
         if (!$dashboard->canViewCurrent() && !$embed) {
-            http_response_code(403);
-            exit();
+            throw new AccessDeniedHttpException();
         }
 
         Session::writeClose();
-        header("Content-Type: application/json; charset=UTF-8");
         $cards = $request_data['cards'];
         unset($request_data['cards']);
         $result = [];
+        Profiler::getInstance()->start('Get cards HTML');
         foreach ($cards as $card) {
             try {
                 $result[$card['card_id']] = $grid->getCardHtml($card['card_id'], array_merge($request_data, $card));
-            } catch (\Throwable $e) {
+            } catch (Throwable $e) {
                 // Send exception to logger without actually exiting.
-                // Use quiet mode to not break JSON result.
-                /** @var \GLPI $GLPI */
-                global $GLPI;
-                $GLPI->getErrorHandler()->handleException($e, true);
+                ErrorHandler::logCaughtException($e);
             }
         }
+        Profiler::getInstance()->stop('Get cards HTML');
         echo json_encode($result);
         break;
 
     case 'display_add_filter':
         if (!$dashboard->canUpdateCurrent()) {
-            http_response_code(403);
-            exit();
+            throw new AccessDeniedHttpException();
         }
 
         $grid->displayFilterForm($_REQUEST);
         break;
     case 'get_dashboard_filters':
         if (!Session::haveRight('dashboard', READ)) {
-            http_response_code(403);
-            exit();
+            throw new AccessDeniedHttpException();
         }
 
         echo $grid->getFiltersSetHtml($_REQUEST['filters'] ?? []);
         break;
     case 'get_filter':
         if (!Session::haveRight('dashboard', READ)) {
-            http_response_code(403);
-            exit();
+            throw new AccessDeniedHttpException();
         }
 
         echo $grid->getFilterHtml($_REQUEST['filter_id']);
@@ -246,8 +265,7 @@ switch ($_REQUEST['action']) {
 
     case 'get_dashboard_items':
         if (!$dashboard->canViewCurrent() && !$embed) {
-            http_response_code(403);
-            exit();
+            throw new AccessDeniedHttpException();
         }
 
         echo $grid->getGridItemsHtml(true, $_REQUEST['embed'] ?? false);
