@@ -8,7 +8,6 @@
  * http://glpi-project.org
  *
  * @copyright 2015-2025 Teclib' and contributors.
- * @copyright 2003-2014 by the INDEPNET Development Team.
  * @licence   https://www.gnu.org/licenses/gpl-3.0.html
  *
  * ---------------------------------------------------------------------
@@ -32,12 +31,10 @@
  *
  * ---------------------------------------------------------------------
  */
-
 /**
- * @var \DBmysql $DB
- * @var \Migration $migration
+ * @var DBmysql $DB
+ * @var Migration $migration
  */
-
 $default_charset = DBConnection::getDefaultCharset();
 $default_collation = DBConnection::getDefaultCollation();
 
@@ -47,7 +44,7 @@ if (!$DB->fieldExists('glpi_authldaps', 'tls_certfile')) {
         'tls_certfile',
         'text',
         [
-            'after'  => 'inventory_domain'
+            'after'  => 'inventory_domain',
         ]
     );
 }
@@ -58,7 +55,7 @@ if (!$DB->fieldExists('glpi_authldaps', 'tls_keyfile')) {
         'tls_keyfile',
         'text',
         [
-            'after'  => 'tls_certfile'
+            'after'  => 'tls_certfile',
         ]
     );
 }
@@ -70,7 +67,7 @@ if (!$DB->fieldExists('glpi_authldaps', 'use_bind')) {
         'bool',
         [
             'after'  => 'tls_keyfile',
-            'value' => 1
+            'value' => 1,
         ]
     );
 }
@@ -82,7 +79,7 @@ if (!$DB->fieldExists('glpi_authldaps', 'timeout')) {
         'int',
         [
             'after'  => 'use_bind',
-            'value' => 10
+            'value' => 10,
         ]
     );
 }
@@ -94,7 +91,7 @@ if (!$DB->fieldExists('glpi_authldapreplicates', 'timeout')) {
         'int',
         [
             'after'  => 'name',
-            'value' => 10
+            'value' => 10,
         ]
     );
 }
