@@ -33,8 +33,6 @@
  * ---------------------------------------------------------------------
  */
 
-use Glpi\Toolbox\Sanitizer;
-
 abstract class AbstractRightsDropdown
 {
     /**
@@ -42,6 +40,8 @@ abstract class AbstractRightsDropdown
      */
     public const LIMIT = 50;
 
+    public const ALL_USERS = "all";
+
     /**
      * To be redefined by subclasses, URL to front file
      *
@@ -52,20 +52,28 @@ abstract class AbstractRightsDropdown
     /**
      * To be redefined by subclasses, specify enabled types
      *
+     * @param array $options Additional options
+     *
      * @return array
      */
-    abstract protected static function getTypes(): array;
+    abstract protected static function getTypes(array $options = []): array;
+
+    protected static function addAllUsersOption(): bool
+    {
+        return false;
+    }
 
     /**
      * Check if a given type is enabled
      *
      * @param string $type Class to check
+     * @param array $options Additional options
      *
      * @return bool
      */
-    protected static function isTypeEnabled(string $type): bool
+    protected static function isTypeEnabled(string $type, array $options = []): bool
     {
-        $types = array_flip(static::getTypes());
+        $types = array_flip(static::getTypes($options));
         return isset($types[$type]);
     }
 
@@ -77,7 +85,7 @@ abstract class AbstractRightsDropdown
      *
      * @return string
      */
-    public static function show(string $name, array $values): string
+    public static function show(string $name, array $values, array $params = []): string
     {
         // Flatten values
         $dropdown_values = [];
@@ -94,12 +102,19 @@ abstract class AbstractRightsDropdown
         $url = static::getAjaxUrl();
 
         // Build params
-        $params = [
+        $params = array_merge([
             'name'        => $name . "[]",
-            'values'      => $dropdown_values,
-            'valuesnames' => self::getValueNames($dropdown_values),
             'multiple'    => true,
-        ];
+            'width'       => '100%',
+        ], $params);
+
+        if ($params['multiple']) {
+            $params['values'] = $dropdown_values;
+            $params['valuesnames'] = static::getValueNames($dropdown_values);
+        } elseif (count($dropdown_values) > 0) {
+            $params['value'] = $dropdown_values[0];
+            $params['valuename'] = static::getValueNames($dropdown_values)[0];
+        }
         return Html::jsAjaxDropdown($params['name'], $field_id, $url, $params);
     }
 
@@ -107,31 +122,42 @@ abstract class AbstractRightsDropdown
      * Get possible data for profiles
      *
      * @param string $text Search string
+     * @param array $options Additional options
      *
      * @return array
      */
-    public static function fetchValues(string $text = ""): array
+    public static function fetchValues(string $text = "", array $options = []): array
     {
         $possible_rights = [];
 
-       // Add profiles if enabled
+        // Add profiles if enabled
         if (self::isTypeEnabled(Profile::getType())) {
             $possible_rights[Profile::getType()] = self::getProfiles($text);
         }
 
-       // Add entities if enabled
+        // Add entities if enabled
         if (self::isTypeEnabled(Entity::getType())) {
             $possible_rights[Entity::getType()] = self::getEntities($text);
         }
 
-       // Add users if enabled
-        if (self::isTypeEnabled(User::getType())) {
-            $possible_rights[User::getType()] = self::getUsers($text);
+        // Add users if enabled
+        if (self::isTypeEnabled(User::getType(), $options)) {
+            $possible_rights[User::getType()] = self::getUsers($text, $options);
         }
 
-       // Add groups if enabled
-        if (self::isTypeEnabled(Group::getType())) {
-            $possible_rights[Group::getType()] = self::getGroups($text);
+        // Add groups if enabled
+        if (self::isTypeEnabled(Group::getType(), $options)) {
+            $possible_rights[Group::getType()] = self::getGroups($text, $options);
+        }
+
+        // Add contacts if enabled
+        if (self::isTypeEnabled(Contact::getType())) {
+            $possible_rights[Contact::getType()] = self::getContacts($text);
+        }
+
+        // Add suppliers if enabled
+        if (self::isTypeEnabled(Supplier::getType())) {
+            $possible_rights[Supplier::getType()] = self::getSuppliers($text);
         }
 
         $results = [];
@@ -151,8 +177,8 @@ abstract class AbstractRightsDropdown
         }
 
         $ret = [
-            'results' => Sanitizer::unsanitize($results),
-            'count' =>  count($results)
+            'results' => $results,
+            'count' =>  count($results),
         ];
 
         return $ret;
@@ -169,13 +195,16 @@ abstract class AbstractRightsDropdown
     {
         return array_map(function ($value) {
             $data = explode("-", $value);
-            $itemtype = getItemtypeForForeignKeyField($data[0]);
             $items_id = $data[1];
-            $item = new $itemtype();
+            $item = getItemForForeignKeyField($data[0]);
 
-            return $itemtype::getTypeName(1) . " - " . Dropdown::getDropdownName(
+            if ($items_id == self::ALL_USERS) {
+                return $item::getTypeName(1) . " - " . __("All users");
+            }
+
+            return $item::getTypeName(1) . " - " . Dropdown::getDropdownName(
                 $item->getTable(),
-                $items_id
+                (int) $items_id
             );
         }, $values);
     }
@@ -191,7 +220,7 @@ abstract class AbstractRightsDropdown
     {
         $profile_item = new Profile();
         $profiles = $profile_item->find([
-            'name' => ["LIKE", "%$text%"]
+            'name' => ["LIKE", "%$text%"],
         ], [], self::LIMIT);
         $profiles_items = [];
         foreach ($profiles as $profile) {
@@ -214,7 +243,7 @@ abstract class AbstractRightsDropdown
         $entity_item = new Entity();
         $entities = $entity_item->find(
             [
-                'name' => ["LIKE", "%$text%"]
+                'name' => ["LIKE", "%$text%"],
             ] + getEntitiesRestrictCriteria(Entity::getTable()),
             [],
             self::LIMIT
@@ -232,13 +261,20 @@ abstract class AbstractRightsDropdown
      * Get possible values for users
      *
      * @param string $text Search string
+     * @param array $options Additional options
      *
      * @return array
      */
-    protected static function getUsers(string $text): array
+    protected static function getUsers(string $text, array $options): array
     {
         $users = User::getSqlSearchResult(false, "all", -1, 0, [], $text, 0, self::LIMIT);
         $users_items = [];
+
+        if (static::addAllUsersOption()) {
+            $new_key = 'users_id-' . self::ALL_USERS;
+            $users_items[$new_key] = __("All users");
+        }
+
         foreach ($users as $user) {
             $new_key = 'users_id-' . $user['id'];
             $users_items[$new_key] = $user['name'];
@@ -251,16 +287,22 @@ abstract class AbstractRightsDropdown
      * Get possible values for groups
      *
      * @param string $text Search string
+     * @param array $options Additional options
      *
      * @return array
      */
-    protected static function getGroups(string $text): array
+    protected static function getGroups(string $text, array $options): array
     {
+        $additional_conditions = [];
+        if (isset($options['group_conditions'])) {
+            $additional_conditions = $options['group_conditions'];
+        }
+
         $group_item = new Group();
         $groups = $group_item->find(
             [
-                'name' => ["LIKE", "%$text%"]
-            ] + getEntitiesRestrictCriteria(Group::getTable()),
+                'name' => ["LIKE", "%$text%"],
+            ] + getEntitiesRestrictCriteria(Group::getTable()) + $additional_conditions,
             [],
             self::LIMIT
         );
@@ -273,6 +315,58 @@ abstract class AbstractRightsDropdown
         return $groups_items;
     }
 
+    /**
+     * Get possible values for contacts
+     *
+     * @param string $text Search string
+     *
+     * @return array
+     */
+    protected static function getContacts(string $text): array
+    {
+        $contact_item = new Contact();
+        $contacts = $contact_item->find(
+            [
+                'name' => ["LIKE", "%$text%"],
+            ] + getEntitiesRestrictCriteria(Contact::getTable()),
+            [],
+            self::LIMIT
+        );
+        $contacts_item = [];
+        foreach ($contacts as $contact) {
+            $new_key = 'contacts_id-' . $contact['id'];
+            $contacts_item[$new_key] = $contact['name'];
+        }
+
+        return $contacts_item;
+    }
+
+    /**
+     * Get possible values for suppliers
+     *
+     * @param string $text Search string
+     *
+     * @return array
+     */
+    protected static function getSuppliers(string $text): array
+    {
+        $supplier_item = new Supplier();
+        $suppliers = $supplier_item->find(
+            [
+                'name' => ["LIKE", "%$text%"],
+            ] + getEntitiesRestrictCriteria(Supplier::getTable()),
+            [],
+            self::LIMIT
+        );
+        $suppliers_item = [];
+        foreach ($suppliers as $supplier) {
+            $new_key = 'suppliers_id-' . $supplier['id'];
+            $suppliers_item[$new_key] = $supplier['name'];
+        }
+
+        return $suppliers_item;
+    }
+
     /**
      * To be used in front files dealing with dropdown input created by
      * static::show()
@@ -302,7 +396,11 @@ abstract class AbstractRightsDropdown
             // Split fkey and ids
             $parsed_values = explode("-", $value);
             $fkey  = $parsed_values[0];
-            $value = (int)$parsed_values[1];
+
+            $value = $parsed_values[1];
+            if (is_numeric($value)) {
+                $value = (int) $value;
+            }
 
             if ($fkey == $class::getForeignKeyField()) {
                 $inflated_values[] = $value;
