@@ -33,11 +33,13 @@
  * ---------------------------------------------------------------------
  */
 
-// Relation between CartridgeItem and PrinterModel
-// since version 0.84
+/**
+ *  Relation between CartridgeItem and PrinterModel
+ *  @since 0.84
+ **/
 class CartridgeItem_PrinterModel extends CommonDBRelation
 {
-   // From CommonDBRelation
+    // From CommonDBRelation
     public static $itemtype_1          = 'CartridgeItem';
     public static $items_id_1          = 'cartridgeitems_id';
 
@@ -58,9 +60,8 @@ class CartridgeItem_PrinterModel extends CommonDBRelation
 
     public static function displayTabContentForItem(CommonGLPI $item, $tabnum = 1, $withtemplate = 0)
     {
-
-        switch ($item->getType()) {
-            case 'CartridgeItem':
+        switch (true) {
+            case $item instanceof CartridgeItem:
                 self::showForCartridgeItem($item);
                 break;
         }
@@ -70,6 +71,9 @@ class CartridgeItem_PrinterModel extends CommonDBRelation
 
     public function getTabNameForItem(CommonGLPI $item, $withtemplate = 0)
     {
+        if (!$item instanceof CommonDBTM) {
+            return '';
+        }
 
         if (!$withtemplate && Printer::canView()) {
             $nb = 0;
@@ -78,7 +82,7 @@ class CartridgeItem_PrinterModel extends CommonDBRelation
                     if ($_SESSION['glpishow_count_on_tabs']) {
                         $nb = self::countForItem($item);
                     }
-                    return self::createTabEntry(PrinterModel::getTypeName(Session::getPluralNumber()), $nb);
+                    return self::createTabEntry(PrinterModel::getTypeName(Session::getPluralNumber()), $nb, $item::getType());
             }
         }
         return '';
@@ -94,8 +98,8 @@ class CartridgeItem_PrinterModel extends CommonDBRelation
      **/
     public static function showForCartridgeItem(CartridgeItem $item)
     {
+        $instID = $item->getID();
 
-        $instID = $item->getField('id');
         if (!$item->can($instID, READ)) {
             return false;
         }
@@ -115,11 +119,11 @@ class CartridgeItem_PrinterModel extends CommonDBRelation
         if ($canedit) {
             echo "<div class='firstbloc'>";
             echo "<form name='printermodel_form$rand' id='printermodel_form$rand' method='post'";
-            echo " action='" . static::getFormURL() . "'>";
+            echo " action='" . htmlescape(static::getFormURL()) . "'>";
 
             echo "<table class='tab_cadre_fixe'>";
             echo "<tr class='tab_bg_1'>";
-            echo "<th colspan='6'>" . __('Add a compatible printer model') . "</th></tr>";
+            echo "<th colspan='6'>" . __s('Add a compatible printer model') . "</th></tr>";
 
             echo "<tr><td class='tab_bg_2 center'>";
             echo "<input type='hidden' name='cartridgeitems_id' value='$instID'>";
@@ -136,9 +140,9 @@ class CartridgeItem_PrinterModel extends CommonDBRelation
             echo "<div class='spaced'>";
             if ($canedit) {
                 $rand     = mt_rand();
-                Html::openMassiveActionsForm('mass' . __CLASS__ . $rand);
+                Html::openMassiveActionsForm('mass' . self::class . $rand);
                 $massiveactionparams = ['num_displayed' => min($_SESSION['glpilist_limit'], count($used)),
-                    'container'     => 'mass' . __CLASS__ . $rand
+                    'container'     => 'mass' . self::class . $rand,
                 ];
                 Html::showMassiveActions($massiveactionparams);
             }
@@ -150,18 +154,18 @@ class CartridgeItem_PrinterModel extends CommonDBRelation
             $header_end    = '';
             if ($canedit) {
                 $header_begin  .= "<th width='10'>";
-                $header_top    .= Html::getCheckAllAsCheckbox('mass' . __CLASS__ . $rand);
-                $header_bottom .= Html::getCheckAllAsCheckbox('mass' . __CLASS__ . $rand);
+                $header_top    .= Html::getCheckAllAsCheckbox('mass' . self::class . $rand);
+                $header_bottom .= Html::getCheckAllAsCheckbox('mass' . self::class . $rand);
                 $header_end    .= "</th>";
             }
-            $header_end .= "<th>" . _n('Model', 'Models', 1) . "</th></tr>";
+            $header_end .= "<th>" . _sn('Model', 'Models', 1) . "</th></tr>";
             echo $header_begin . $header_top . $header_end;
 
             foreach ($datas as $data) {
                 echo "<tr class='tab_bg_1'>";
                 if ($canedit) {
                     echo "<td width='10'>";
-                    Html::showMassiveActionCheckBox(__CLASS__, $data["linkid"]);
+                    Html::showMassiveActionCheckBox(self::class, $data["linkid"]);
                     echo "</td>";
                 }
                 $opt = [
@@ -171,11 +175,11 @@ class CartridgeItem_PrinterModel extends CommonDBRelation
                             'field'      => 40, // printer model
                             'searchtype' => 'equals',
                             'value'      => $data["id"],
-                        ]
-                    ]
+                        ],
+                    ],
                 ];
-                $url = Printer::getSearchURL() . "?" . Toolbox::append_params($opt, '&amp;');
-                echo "<td class='center'><a href='" . $url . "'>" . $data["name"] . "</a></td>";
+                $url = Printer::getSearchURL() . "?" . Toolbox::append_params($opt);
+                echo "<td class='center'><a href='" . htmlescape($url) . "'>" . htmlescape($data["name"]) . "</a></td>";
                 echo "</tr>";
             }
             echo $header_begin . $header_bottom . $header_end;
@@ -187,7 +191,7 @@ class CartridgeItem_PrinterModel extends CommonDBRelation
             }
             echo "</div>";
         } else {
-            echo "<p class='center b'>" . __('No item found') . "</p>";
+            echo "<p class='center b'>" . __s('No results found') . "</p>";
         }
     }
 }
