@@ -32,9 +32,13 @@
  *
  * ---------------------------------------------------------------------
  */
-
 use Glpi\Application\View\TemplateRenderer;
+use Glpi\DBAL\QueryExpression;
+use Glpi\DBAL\QueryFunction;
 use Glpi\Features\AssetImage;
+use Glpi\Features\AssignableItem;
+use Glpi\Features\AssignableItemInterface;
+use Glpi\Features\Clonable;
 
 //!  ConsumableItem Class
 /**
@@ -42,11 +46,15 @@ use Glpi\Features\AssetImage;
  * @see Consumable
  * @author Julien Dombre
  */
-class ConsumableItem extends CommonDBTM
+class ConsumableItem extends CommonDBTM implements AssignableItemInterface
 {
-    use Glpi\Features\Clonable;
+    use Clonable;
 
     use AssetImage;
+    use AssignableItem {
+        prepareInputForAdd as prepareInputForAddAssignableItem;
+        prepareInputForUpdate as prepareInputForUpdateAssignableItem;
+    }
 
     // From CommonDBTM
     protected static $forward_entity_to = ['Consumable', 'Infocom'];
@@ -57,7 +65,10 @@ class ConsumableItem extends CommonDBTM
 
     public function getCloneRelations(): array
     {
-        return [];
+        return [
+            Infocom::class,
+            ManualLink::class,
+        ];
     }
 
     public static function getTypeName($nb = 0)
@@ -65,26 +76,21 @@ class ConsumableItem extends CommonDBTM
         return _n('Consumable model', 'Consumable models', $nb);
     }
 
-
     public static function getMenuName()
     {
         return Consumable::getTypeName(Session::getPluralNumber());
     }
 
-
     public static function getAdditionalMenuLinks()
     {
-
         if (static::canView()) {
             return ['summary' => '/front/consumableitem.php?synthese=yes'];
         }
         return false;
     }
 
-
     public function getPostAdditionalInfosForName()
     {
-
         if (isset($this->fields["ref"]) && !empty($this->fields["ref"])) {
             return $this->fields["ref"];
         }
@@ -93,19 +99,24 @@ class ConsumableItem extends CommonDBTM
 
     public function prepareInputForAdd($input)
     {
-        $input = parent::prepareInputForAdd($input);
+        $input = $this->prepareInputForAddAssignableItem($input);
+        if ($input === false) {
+            return false;
+        }
         return $this->managePictures($input);
     }
 
     public function prepareInputForUpdate($input)
     {
-        $input = parent::prepareInputForUpdate($input);
+        $input = $this->prepareInputForUpdateAssignableItem($input);
+        if ($input === false) {
+            return false;
+        }
         return $this->managePictures($input);
     }
 
     public function cleanDBonPurge()
     {
-
         $this->deleteChildrenAndRelationsFromDb(
             [
                 Consumable::class,
@@ -114,13 +125,11 @@ class ConsumableItem extends CommonDBTM
 
         // Alert does not extends CommonDBConnexity
         $alert = new Alert();
-        $alert->cleanDBonItemDelete($this->getType(), $this->fields['id']);
+        $alert->cleanDBonItemDelete(static::class, $this->fields['id']);
     }
 
-
     public function post_getEmpty()
     {
-
         if (isset($_SESSION['glpiactive_entity'])) {
             $this->fields["alarm_threshold"] = Entity::getUsedConfig(
                 "consumables_alert_repeat",
@@ -131,30 +140,29 @@ class ConsumableItem extends CommonDBTM
         }
     }
 
-
     public function defineTabs($options = [])
     {
-
         $ong = [];
         $this->addDefaultFormTab($ong);
-        $this->addStandardTab('Consumable', $ong, $options);
-        $this->addStandardTab('Infocom', $ong, $options);
-        $this->addStandardTab('Document_Item', $ong, $options);
-        $this->addStandardTab('ManualLink', $ong, $options);
-        $this->addStandardTab('Notepad', $ong, $options);
-        $this->addStandardTab('Log', $ong, $options);
+        $this->addStandardTab(Consumable::class, $ong, $options);
+        $this->addStandardTab(Infocom::class, $ong, $options);
+        $this->addStandardTab(Document_Item::class, $ong, $options);
+        $this->addStandardTab(ManualLink::class, $ong, $options);
+        $this->addStandardTab(Notepad::class, $ong, $options);
+        $this->addStandardTab(Log::class, $ong, $options);
 
         return $ong;
     }
 
-
     public function rawSearchOptions()
     {
+        global $DB;
+
         $tab = parent::rawSearchOptions();
 
         $tab[] = [
             'id'                 => '2',
-            'table'              => $this->getTable(),
+            'table'              => static::getTable(),
             'field'              => 'id',
             'name'               => __('ID'),
             'datatype'           => 'number',
@@ -163,7 +171,7 @@ class ConsumableItem extends CommonDBTM
 
         $tab[] = [
             'id'                 => '34',
-            'table'              => $this->getTable(),
+            'table'              => static::getTable(),
             'field'              => 'ref',
             'name'               => __('Reference'),
             'datatype'           => 'string',
@@ -171,7 +179,7 @@ class ConsumableItem extends CommonDBTM
 
         $tab[] = [
             'id'                 => '6',
-            'table'              => $this->getTable(),
+            'table'              => static::getTable(),
             'field'              => 'otherserial',
             'name'               => __('Inventory number'),
             'datatype'           => 'string',
@@ -195,7 +203,7 @@ class ConsumableItem extends CommonDBTM
 
         $tab[] = [
             'id'                 => '9',
-            'table'              => $this->getTable(),
+            'table'              => static::getTable(),
             'field'              => '_virtual',
             'linkfield'          => '_virtual',
             'name'               => _n('Consumable', 'Consumables', Session::getPluralNumber()),
@@ -209,31 +217,37 @@ class ConsumableItem extends CommonDBTM
         $tab[] = [
             'id'                 => '17',
             'table'              => 'glpi_consumables',
-            'field'              => 'id',
+            'field'              => 'date_out',
             'name'               => __('Number of used consumables'),
-            'datatype'           => 'count',
+            'datatype'           => 'number',
             'forcegroupby'       => true,
             'usehaving'          => true,
             'massiveaction'      => false,
+            'nometa'             => true,
             'joinparams'         => [
                 'jointype'           => 'child',
-                'condition'          => ['NOT' => ['NEWTABLE.date_out' => null]],
             ],
+            'computation' => new QueryExpression(
+                expression: QueryFunction::sum(new QueryExpression("CASE WHEN " . $DB::quoteName('TABLE.date_out') . " IS NULL THEN 1 ELSE 0 END"))
+            ),
         ];
 
         $tab[] = [
             'id'                 => '19',
             'table'              => 'glpi_consumables',
-            'field'              => 'id',
+            'field'              => 'date_out',
             'name'               => __('Number of new consumables'),
-            'datatype'           => 'count',
+            'datatype'           => 'number',
             'forcegroupby'       => true,
             'usehaving'          => true,
             'massiveaction'      => false,
+            'nometa'             => true,
             'joinparams'         => [
                 'jointype'           => 'child',
-                'condition'          => ['NEWTABLE.date_out' => null],
             ],
+            'computation' => new QueryExpression(
+                expression: QueryFunction::sum(new QueryExpression("CASE WHEN " . $DB::quoteName('TABLE.date_out') . " IS NOT NULL THEN 1 ELSE 0 END"))
+            ),
         ];
 
         $tab = array_merge($tab, Location::rawSearchOptionsToAdd());
@@ -252,15 +266,26 @@ class ConsumableItem extends CommonDBTM
             'id'                 => '49',
             'table'              => 'glpi_groups',
             'field'              => 'completename',
-            'linkfield'          => 'groups_id_tech',
+            'linkfield'          => 'groups_id',
             'name'               => __('Group in charge'),
             'condition'          => ['is_assign' => 1],
+            'joinparams'         => [
+                'beforejoin'         => [
+                    'table'              => 'glpi_groups_items',
+                    'joinparams'         => [
+                        'jointype'           => 'itemtype_item',
+                        'condition'          => ['NEWTABLE.type' => Group_Item::GROUP_TYPE_TECH],
+                    ],
+                ],
+            ],
+            'forcegroupby'       => true,
+            'massiveaction'      => false,
             'datatype'           => 'dropdown',
         ];
 
         $tab[] = [
             'id'                 => '8',
-            'table'              => $this->getTable(),
+            'table'              => static::getTable(),
             'field'              => 'alarm_threshold',
             'name'               => __('Alert threshold'),
             'datatype'           => 'number',
@@ -271,9 +296,9 @@ class ConsumableItem extends CommonDBTM
 
         $tab[] = [
             'id'                 => '16',
-            'table'              => $this->getTable(),
+            'table'              => static::getTable(),
             'field'              => 'comment',
-            'name'               => __('Comments'),
+            'name'               => _n('Comment', 'Comments', Session::getPluralNumber()),
             'datatype'           => 'text',
         ];
 
@@ -291,33 +316,26 @@ class ConsumableItem extends CommonDBTM
         return $tab;
     }
 
-
     public static function cronInfo($name)
     {
         return ['description' => __('Send alarms on consumables')];
     }
 
-
     /**
      * Cron action on consumables : alert if a stock is behind the threshold
      *
      * @param CronTask|null $task to log, if NULL display (default NULL)
      *
      * @return integer 0 : nothing to do 1 : done with success
+     * @used-by CronTask
      **/
     public static function cronConsumable(?CronTask $task = null)
     {
-        /**
-         * @var array $CFG_GLPI
-         * @var \DBmysql $DB
-         */
         global $CFG_GLPI, $DB;
 
         $cron_status = 1;
 
         if ($CFG_GLPI["use_notifications"]) {
-            $message = [];
-            $items   = [];
             $alert   = new Alert();
 
             foreach (Entity::getEntitiesToNotify('consumables_alert_repeat') as $entity => $repeat) {
@@ -332,7 +350,7 @@ class ConsumableItem extends CommonDBTM
                             'glpi_alerts.id AS alertID',
                             'glpi_alerts.date',
                         ],
-                        'FROM'      => ConsumableItem::getTable(),
+                        'FROM'      => self::getTable(),
                         'LEFT JOIN' => [
                             'glpi_alerts' => [
                                 'FKEY' => [
@@ -350,14 +368,22 @@ class ConsumableItem extends CommonDBTM
                             'glpi_consumableitems.entities_id'     => $entity,
                             'OR'                                  => [
                                 ['glpi_alerts.date' => null],
-                                ['glpi_alerts.date' => ['<', new QueryExpression('CURRENT_TIMESTAMP() - INTERVAL ' . $repeat . ' second')]],
+                                [
+                                    'glpi_alerts.date' => ['<',
+                                        QueryFunction::dateSub(
+                                            date: QueryFunction::now(),
+                                            interval: $repeat,
+                                            interval_unit: 'SECOND'
+                                        ),
+                                    ],
+                                ],
                             ],
                         ],
                     ]
                 );
 
-                $message = "";
-                $items   = [];
+                $messages = [];
+                $items    = [];
 
                 foreach ($alerts_result as $consumable) {
                     if (
@@ -366,13 +392,12 @@ class ConsumableItem extends CommonDBTM
                     ) {
                         // define message alert
                         //TRANS: %1$s is the consumable name, %2$s its reference, %3$d the remaining number
-                        $message .= sprintf(
+                        $messages[] = sprintf(
                             __('Threshold of alarm reached for the type of consumable: %1$s - Reference %2$s - Remaining %3$d'),
                             $consumable['name'],
                             $consumable['ref'],
                             $unused
                         );
-                        $message .= '<br>';
 
                         $items[$consumable["consID"]] = $consumable;
 
@@ -383,7 +408,7 @@ class ConsumableItem extends CommonDBTM
                     }
                 }
 
-                if (!empty($items)) {
+                if ($items !== []) {
                     $options = [
                         'entities_id' => $entity,
                         'items'       => $items,
@@ -391,17 +416,18 @@ class ConsumableItem extends CommonDBTM
 
                     if (NotificationEvent::raiseEvent('alert', new ConsumableItem(), $options)) {
                         if ($task) {
-                            $task->log(Dropdown::getDropdownName(
-                                "glpi_entities",
-                                $entity
-                            ) . " :  $message\n");
+                            $task->log(
+                                Dropdown::getDropdownName("glpi_entities", $entity)
+                                . " : "
+                                . implode("\n", $messages)
+                            );
                             $task->addVolume(1);
                         } else {
-                            Session::addMessageAfterRedirect(Dropdown::getDropdownName(
-                                "glpi_entities",
-                                $entity
-                            ) .
-                                                     " :  $message");
+                            Session::addMessageAfterRedirect(
+                                htmlescape(Dropdown::getDropdownName("glpi_entities", $entity))
+                                . " : "
+                                . implode('<br>', array_map('htmlescape', $messages))
+                            );
                         }
 
                         $input = [
@@ -410,7 +436,7 @@ class ConsumableItem extends CommonDBTM
                         ];
 
                         // add alerts
-                        foreach ($items as $ID => $consumable) {
+                        foreach (array_keys($items) as $ID) {
                             $input["items_id"] = $ID;
                             $alert->add($input);
                             unset($alert->fields['id']);
@@ -422,7 +448,7 @@ class ConsumableItem extends CommonDBTM
                         if ($task) {
                             $task->log($msg);
                         } else {
-                            Session::addMessageAfterRedirect($msg, false, ERROR);
+                            Session::addMessageAfterRedirect(htmlescape($msg), false, ERROR);
                         }
                     }
                 }
@@ -431,43 +457,11 @@ class ConsumableItem extends CommonDBTM
         return $cron_status;
     }
 
-
     public function getEvents()
     {
         return ['alert' => __('Send alarms on consumables')];
     }
 
-
-    /**
-     * Display debug information for current object
-     **/
-    public function showDebug()
-    {
-
-        // see query_alert in cronConsumable()
-        $item = ['consID'    => $this->fields['id'],
-            'entity'    => $this->fields['entities_id'],
-            'ref'       => $this->fields['ref'],
-            'name'      => $this->fields['name'],
-            'threshold' => $this->fields['alarm_threshold'],
-        ];
-
-        $options = [];
-        $options['entities_id'] = $this->getEntityID();
-        $options['items']       = [$item];
-        NotificationEvent::debugEvent($this, $options);
-    }
-
-
-    public function canUpdateItem()
-    {
-
-        if (!$this->checkEntity(true)) { //check entities recursively
-            return false;
-        }
-        return true;
-    }
-
     public function showForm($ID, array $options = [])
     {
         $this->initForm($ID, $options);
