@@ -7,7 +7,7 @@
  *
  * http://glpi-project.org
  *
- * @copyright 2015-2024 Teclib' and contributors.
+ * @copyright 2015-2025 Teclib' and contributors.
  * @copyright 2003-2014 by the INDEPNET Development Team.
  * @licence   https://www.gnu.org/licenses/gpl-3.0.html
  *
@@ -33,8 +33,12 @@
  * ---------------------------------------------------------------------
  */
 
+use Safe\Exceptions\PcreException;
+
+use function Safe\preg_match;
+
 /**
- * Create an abstration layer for any kind of internet label
+ * Create an abstraction layer for any kind of internet label
  */
 
 
@@ -42,19 +46,23 @@
 /// Since version 0.84
 abstract class FQDNLabel extends CommonDBChild
 {
-   // Inherits from CommonDBChild as it must be attached to a specific element
-   // (NetworkName, NetworkPort, ...)
+    // Inherits from CommonDBChild as it must be attached to a specific element
+    // (NetworkName, NetworkPort, ...)
 
     public function getInternetName()
     {
 
-       // get the full computer name of the current object (for instance : forge.indepnet.net)
+        // get the full computer name of the current object (for instance : forge.indepnet.net)
         return self::getInternetNameFromLabelAndDomainID(
             $this->fields["name"],
             $this->fields["fqdns_id"]
         );
     }
 
+    public static function getIcon()
+    {
+        return 'ti ti-signature';
+    }
 
     /**
      * Get the internet name from a label and a domain ID
@@ -84,22 +92,24 @@ abstract class FQDNLabel extends CommonDBChild
      **/
     public static function checkFQDNLabel($label)
     {
-
-        if (strlen($label) == 1) {
-            if (!preg_match("/^[0-9A-Za-z]$/", $label, $regs)) {
-                return false;
-            }
-        } else {
-            $fqdn_regex = "/^(?!-)[A-Za-z0-9-]{1,63}(?<!-)$/";
-            if (!preg_match($fqdn_regex, $label, $regs)) {
-               //check also Internationalized domain name
-                $idn = idn_to_ascii($label);
-                if (!preg_match($fqdn_regex, $idn, $regs)) {
+        try {
+            if (strlen($label) == 1) {
+                if (!preg_match("/^[0-9A-Za-z]$/", $label, $regs)) {
                     return false;
                 }
+            } else {
+                $fqdn_regex = "/^(?!-)[A-Za-z0-9-]{1,63}(?<!-)$/";
+                if (!preg_match($fqdn_regex, $label, $regs)) {
+                    //check also Internationalized domain name
+                    $idn = idn_to_ascii($label);
+                    if (!preg_match($fqdn_regex, $idn, $regs)) {
+                        return false;
+                    }
+                }
             }
+        } catch (PcreException $e) {
+            return false;
         }
-
         return true;
     }
 
@@ -110,16 +120,16 @@ abstract class FQDNLabel extends CommonDBChild
     public function prepareLabelInput($input)
     {
         if (isset($input['name']) && !empty($input['name'])) {
-           // Empty names are allowed
+            // Empty names are allowed
 
             $input['name'] = strtolower($input['name']);
 
-           // Before adding a name, we must unsure its is valid : it conforms to RFC
+            // Before adding a name, we must unsure its is valid : it conforms to RFC
             if (!self::checkFQDNLabel($input['name'])) {
-                Session::addMessageAfterRedirect(sprintf(
+                Session::addMessageAfterRedirect(htmlescape(sprintf(
                     __('Invalid internet name: %s'),
                     $input['name']
-                ), false, ERROR);
+                )), false, ERROR);
                 return false;
             }
         }
@@ -133,11 +143,11 @@ abstract class FQDNLabel extends CommonDBChild
     public function prepareIPNetworkFromInput($input)
     {
 
-       //getIPNetwork from IPV4 if not set
-        if (!isset($input['ipnetworks_id']) || (isset($input['ipnetworks_id']) && $input['ipnetworks_id'] == 0 )) {
+        //getIPNetwork from IPV4 if not set
+        if (!isset($input['ipnetworks_id']) || (isset($input['ipnetworks_id']) && $input['ipnetworks_id'] == 0)) {
             if (isset($input['_ipaddresses'])) {
                 foreach ($input['_ipaddresses'] as $value) {
-                   //if its an ipv4, find it's IPNetwork
+                    //if its an ipv4, find it's IPNetwork
                     if (filter_var($value, FILTER_VALIDATE_IP, FILTER_FLAG_IPV4)) {
                         // get first IPNetwork because :
                         // see IPNetwork::searchNetworks
@@ -182,7 +192,6 @@ abstract class FQDNLabel extends CommonDBChild
      **/
     public static function getIDsByLabelAndFQDNID($label, $fqdns_id, $wildcard_search = false)
     {
-        /** @var \DBmysql $DB */
         global $DB;
 
         $label = strtolower($label);
@@ -200,13 +209,13 @@ abstract class FQDNLabel extends CommonDBChild
         $IDs = [];
         foreach (
             ['NetworkName'  => 'glpi_networknames',
-                'NetworkAlias' => 'glpi_networkaliases'
+                'NetworkAlias' => 'glpi_networkaliases',
             ] as $class => $table
         ) {
             $criteria = [
                 'SELECT' => 'id',
                 'FROM'   => $table,
-                'WHERE'  => ['name' => $relation]
+                'WHERE'  => ['name' => $relation],
             ];
 
             if (
@@ -293,7 +302,7 @@ abstract class FQDNLabel extends CommonDBChild
     {
 
         $labels_with_items = self::getItemsByFQDN($value);
-       // Filter : Do not keep ip not linked to asset
+        // Filter : Do not keep ip not linked to asset
         if (count($labels_with_items)) {
             foreach ($labels_with_items as $key => $tab) {
                 if (
@@ -310,12 +319,12 @@ abstract class FQDNLabel extends CommonDBChild
         }
 
         if (count($labels_with_items)) {
-           // Get the first item that is matching entity
+            // Get the first item that is matching entity
             foreach ($labels_with_items as $items) {
                 foreach ($items as $item) {
                     if ($item->getEntityID() == $entity) {
                         $result = ["id"       => $item->getID(),
-                            "itemtype" => $item->getType()
+                            "itemtype" => $item->getType(),
                         ];
                         unset($labels_with_items);
                         return $result;
