@@ -33,6 +33,8 @@
  * ---------------------------------------------------------------------
  */
 
+use Glpi\DBAL\QueryExpression;
+
 /**
  * Class IPAddress_IPNetwork : Connection between IPAddress and IPNetwork
  *
@@ -40,7 +42,7 @@
  **/
 class IPAddress_IPNetwork extends CommonDBRelation
 {
-   // From CommonDBRelation
+    // From CommonDBRelation
     public static $itemtype_1 = 'IPAddress';
     public static $items_id_1 = 'ipaddresses_id';
 
@@ -55,32 +57,31 @@ class IPAddress_IPNetwork extends CommonDBRelation
      **/
     public static function linkIPAddressFromIPNetwork(IPNetwork $network)
     {
-        /** @var \DBmysql $DB */
         global $DB;
 
         $linkObject    = new self();
         $linkTable     = $linkObject->getTable();
         $ipnetworks_id = $network->getID();
 
-       // First, remove all links of the current Network
+        // First, remove all links of the current Network
         $iterator = $DB->request([
             'SELECT' => 'id',
             'FROM'   => $linkTable,
-            'WHERE'  => ['ipnetworks_id' => $ipnetworks_id]
+            'WHERE'  => ['ipnetworks_id' => $ipnetworks_id],
         ]);
         foreach ($iterator as $link) {
             $linkObject->delete(['id' => $link['id']]);
         }
 
-       // Then, look each IP address contained inside current Network
+        // Then, look each IP address contained inside current Network
         $iterator = $DB->request([
             'SELECT' => [
-                new \QueryExpression($DB->quoteValue($ipnetworks_id) . ' AS ' . $DB->quoteName('ipnetworks_id')),
-                'id AS ipaddresses_id'
+                new QueryExpression($DB->quoteValue($ipnetworks_id) . ' AS ' . $DB->quoteName('ipnetworks_id')),
+                'id AS ipaddresses_id',
             ],
             'FROM'   => 'glpi_ipaddresses',
             'WHERE'  => $network->getCriteriaForMatchingElement('glpi_ipaddresses', 'binary', 'version'),
-            'GROUP'  => 'id'
+            'GROUP'  => 'id',
         ]);
         foreach ($iterator as $link) {
             $linkObject->add($link);
@@ -100,7 +101,7 @@ class IPAddress_IPNetwork extends CommonDBRelation
         $entity         = $ipaddress->getEntityID();
         $ipnetworks_ids = IPNetwork::searchNetworksContainingIP($ipaddress, $entity);
         if ($ipnetworks_ids !== false) {
-           // Beware that invalid IPaddresses don't have any valid address !
+            // Beware that invalid IPaddresses don't have any valid address !
             $entity = $ipaddress->getEntityID();
             foreach (IPNetwork::searchNetworksContainingIP($ipaddress, $entity) as $ipnetworks_id) {
                 $input['ipnetworks_id'] = $ipnetworks_id;
