@@ -33,6 +33,8 @@
  * ---------------------------------------------------------------------
  */
 
+use Glpi\Application\View\TemplateRenderer;
+
 /**
  * Store ports connections log
  *
@@ -45,34 +47,18 @@ class NetworkPortConnectionLog extends CommonDBChild
     public static $items_id        = 'networkports_id';
     public $dohistory              = false;
 
-
-    /**
-     * Get name of this type by language of the user connected
-     *
-     * @param integer $nb number of elements
-     *
-     * @return string name of this type
-     */
     public static function getTypeName($nb = 0)
     {
         return __('Port connection history');
     }
 
-    /**
-     * Get the tab name used for item
-     *
-     * @param CommonGLPI $item the item object
-     * @param integer $withtemplate 1 if is a template form
-     * @return string|array name of the tab
-     */
     public function getTabNameForItem(CommonGLPI $item, $withtemplate = 0)
     {
-
         $array_ret = [];
 
-        if ($item->getType() == 'NetworkPort') {
+        if ($item::class === NetworkPort::class) {
             $cnt = countElementsInTable([static::getTable()], $this->getCriteria($item));
-            $array_ret[] = self::createTabEntry(self::getTypeName(Session::getPluralNumber()), $cnt);
+            $array_ret[] = self::createTabEntry(self::getTypeName(Session::getPluralNumber()), $cnt, $item::class);
         }
         return $array_ret;
     }
@@ -82,22 +68,14 @@ class NetworkPortConnectionLog extends CommonDBChild
         return [
             'OR' => [
                 'networkports_id_source'      => $netport->fields['id'],
-                'networkports_id_destination' => $netport->fields['id']
-            ]
+                'networkports_id_destination' => $netport->fields['id'],
+            ],
         ];
     }
 
-    /**
-     * Display the content of the tab
-     *
-     * @param CommonGLPI $item
-     * @param integer $tabnum number of the tab to display
-     * @param integer $withtemplate 1 if is a template form
-     * @return boolean
-     */
     public static function displayTabContentForItem(CommonGLPI $item, $tabnum = 1, $withtemplate = 0)
     {
-        if (get_class($item) == NetworkPort::class && $item->getID() > 0) {
+        if ($item::class === NetworkPort::class && $item->getID() > 0) {
             $connectionlog = new self();
             $connectionlog->showForItem($item);
             return true;
@@ -107,69 +85,78 @@ class NetworkPortConnectionLog extends CommonDBChild
 
     public function showForItem(NetworkPort $netport, $user_filters = [])
     {
-        /** @var \DBmysql $DB */
         global $DB;
 
         $iterator = $DB->request([
-            'FROM'   => $this->getTable(),
-            'WHERE'  => $this->getCriteria($netport)
+            'FROM'   => static::getTable(),
+            'WHERE'  => $this->getCriteria($netport),
         ]);
 
-        echo "<table class='tab_cadre_fixehov'>";
-        echo "<thead><tr>";
-        echo "<th>" . _n('State', 'States', 1)  . "</th>";
-        echo "<th>" . _n('Date', 'Dates', 1)  . "</th>";
-        echo "<th>" . __('Connected item')  . "</th>";
-        echo "</tr></thead>";
-
-        echo "<tbody>";
-
-        if (!count($iterator)) {
-            echo "<tr><td colspan='4' class='center'>" . __('No result found')  . "</td></tr>";
-        }
-
+        $entries = [];
         foreach ($iterator as $row) {
-            echo "<tr>";
-            echo "<td>";
-
-            if ($row['connected'] == 1) {
-                $co_class = 'fa-link netport green';
-                $title = __('Connected');
+            if ($row['connected'] === 1) {
+                $co_class = 'ti-link netport text-success';
+                $title = __s('Connected');
             } else {
-                $co_class = 'fa-unlink netport red';
-                $title = __('Not connected');
+                $co_class = 'ti-unlink netport text-danger';
+                $title = __s('Not connected');
             }
-            echo "<i class='fas $co_class' title='$title'></i> <span class='sr-only'>$title</span>";
-            echo "</td>";
-            echo "<td>" . $row['date']  . "</td>";
-            echo "<td>";
 
-            $is_source = $netport->fields['id'] == $row['networkports_id_source'];
+            $is_source = $netport->fields['id'] === $row['networkports_id_source'];
             $netports_id = $row[($is_source ? 'networkports_id_destination' : 'networkports_id_source')];
 
             $cport = new NetworkPort();
             if ($cport->getFromDB($netports_id)) {
-                $citem = new $cport->fields["itemtype"]();
+                $citem = getItemForItemtype($cport->fields["itemtype"]);
                 $citem->getFromDB($cport->fields["items_id"]);
 
                 $cport_link = sprintf(
-                    "<a href='%1\$s'>%2\$s</a>",
-                    $cport->getFormURLWithID($cport->fields['id']),
-                    (trim($cport->fields['name']) == '' ? __('Without name') : $cport->fields['name'])
+                    '<a href="%1$s">%2$s</a>',
+                    htmlescape($cport::getFormURLWithID($cport->fields['id'])),
+                    htmlescape(trim($cport->fields['name']) === '' ? __('Without name') : $cport->fields['name'])
                 );
 
-                echo sprintf(
-                    '%1$s on %2$s',
-                    $cport_link,
-                    $citem->getLink()
-                );
-            } else if ($row['connected'] == 1) {
-                echo __('No longer exists in database');
+                $entries = [
+                    'status' => '<i class="ti ' . $co_class . '" title="' . $title . '"></i>',
+                    'date' => $row['date'],
+                    'connected_item' => sprintf(
+                        __s('%1$s on %2$s'),
+                        $cport_link,
+                        $citem->getLink()
+                    ),
+                ];
+            } elseif ($row['connected'] === 1) {
+                $entries[] = [
+                    'status' => __s('No longer exists in database'),
+                    'date' => $row['date'],
+                    'connected_item' => __s('Unknown'),
+                ];
             }
-
-            echo "</td>";
-            echo "</tr>";
         }
-        echo "</tbody>";
+
+        TemplateRenderer::getInstance()->display('components/datatable.html.twig', [
+            'is_tab' => true,
+            'nofilter' => true,
+            'nosort' => true,
+            'columns' => [
+                'status' => _n('State', 'States', 1),
+                'date' => _n('Date', 'Dates', 1),
+                'connected_item' => __('Connected item'),
+            ],
+            'formatters' => [
+                'status' => 'raw_html',
+                'date' => 'datetime',
+                'connected_item' => 'raw_html',
+            ],
+            'entries' => $entries,
+            'total_number' => count($entries),
+            'filtered_number' => count($entries),
+            'showmassiveactions' => false,
+        ]);
+    }
+
+    public static function getIcon()
+    {
+        return 'ti ti-history';
     }
 }
