@@ -32,17 +32,19 @@
  *
  * ---------------------------------------------------------------------
  */
-
+use Glpi\Features\Inventoriable;
+use Glpi\Inventory\Inventory;
 use Glpi\Inventory\Request;
+use Glpi\Search\DefaultSearchRequestInterface;
 
 /**
  * Equipments refused from inventory
  */
-class RefusedEquipment extends CommonDBTM
+class RefusedEquipment extends CommonDBTM implements DefaultSearchRequestInterface
 {
-    use Glpi\Features\Inventoriable;
+    use Inventoriable;
 
-   // From CommonDBTM
+    // From CommonDBTM
     public $dohistory                   = true;
     public static $rightname                   = 'refusedequipment';
 
@@ -51,6 +53,11 @@ class RefusedEquipment extends CommonDBTM
         return _n('Equipment refused by rules log', 'Equipments refused by rules log', $nb);
     }
 
+    public static function getSectorizedDetails(): array
+    {
+        return ['admin', Inventory::class, self::class];
+    }
+
     public function rawSearchOptions()
     {
         $tab = parent::rawSearchOptions();
@@ -67,7 +74,7 @@ class RefusedEquipment extends CommonDBTM
 
         $tab[] = [
             'id'            => '3',
-            'table'         => $this->getTable(),
+            'table'         => static::getTable(),
             'field'         => 'date_creation',
             'name'          => _n('Date', 'Dates', 1),
             'datatype'      => 'datetime',
@@ -76,7 +83,7 @@ class RefusedEquipment extends CommonDBTM
 
         $tab[] = [
             'id'            => '4',
-            'table'         => $this->getTable(),
+            'table'         => static::getTable(),
             'field'         => 'itemtype',
             'name'          => __('Item type'),
             'massiveaction' => false,
@@ -94,7 +101,7 @@ class RefusedEquipment extends CommonDBTM
 
         $tab[] = [
             'id'            => '6',
-            'table'         => $this->getTable(),
+            'table'         => static::getTable(),
             'field'         => 'serial',
             'name'          => __('Serial number'),
             'datatype'      => 'string',
@@ -103,7 +110,7 @@ class RefusedEquipment extends CommonDBTM
 
         $tab[] = [
             'id'            => '7',
-            'table'         => $this->getTable(),
+            'table'         => static::getTable(),
             'field'         => 'uuid',
             'name'          => __('UUID'),
             'datatype'      => 'string',
@@ -112,7 +119,7 @@ class RefusedEquipment extends CommonDBTM
 
         $tab[] = [
             'id'            => '8',
-            'table'         => $this->getTable(),
+            'table'         => static::getTable(),
             'field'         => 'ip',
             'name'          => __('IP'),
             'datatype'      => 'text',
@@ -121,7 +128,7 @@ class RefusedEquipment extends CommonDBTM
 
         $tab[] = [
             'id'            => '9',
-            'table'         => $this->getTable(),
+            'table'         => static::getTable(),
             'field'         => 'mac',
             'name'          => __('MAC'),
             'datatype'      => 'text',
@@ -130,7 +137,7 @@ class RefusedEquipment extends CommonDBTM
 
         $tab[] = [
             'id'            => '10',
-            'table'         => $this->getTable(),
+            'table'         => static::getTable(),
             'field'         => 'method',
             'name'          => __('Method'),
             'datatype'      => 'string',
@@ -150,16 +157,12 @@ class RefusedEquipment extends CommonDBTM
         return $tab;
     }
 
-    /**
-     * Get search parameters for default search / display list
-     *
-     * @return array
-     */
-    public static function getDefaultSearchRequest()
+    #[Override]
+    public static function getDefaultSearchRequest(): array
     {
         return [
             'sort'  => 3, //date SO
-            'order' => 'DESC'
+            'order' => 'DESC',
         ];
     }
 
@@ -170,7 +173,6 @@ class RefusedEquipment extends CommonDBTM
 
     public function showForm($ID, array $options = [])
     {
-        /** @var array $CFG_GLPI */
         global $CFG_GLPI;
 
         $this->initForm($ID, $options);
@@ -179,27 +181,27 @@ class RefusedEquipment extends CommonDBTM
         echo "<tr class='tab_bg_1'>";
 
         $itemtype = $this->fields['itemtype'];
-        echo "<th>" .  __('Item type') . "</th>";
-        echo "<td>" . $itemtype::getTypeName(1)  . "</td>";
+        echo "<th>" . __s('Item type') . "</th>";
+        echo "<td>" . htmlescape($itemtype::getTypeName(1)) . "</td>";
 
-        echo "<th>" . __('Name') . "</th>";
-        echo "<td>" . $this->getName()  . "</td>";
+        echo "<th>" . __s('Name') . "</th>";
+        echo "<td>" . htmlescape($this->getName()) . "</td>";
 
         echo "</tr>";
         echo "<tr class='tab_bg_1'>";
 
-        echo "<th>" .  __('Serial') . "</th>";
-        echo "<td>" . $this->fields['serial']  . "</td>";
+        echo "<th>" . __s('Serial') . "</th>";
+        echo "<td>" . htmlescape($this->fields['serial']) . "</td>";
 
-        echo "<th>" .  __('UUID') . "</th>";
-        echo "<td>" . $this->fields['uuid']  . "</td>";
+        echo "<th>" . __s('UUID') . "</th>";
+        echo "<td>" . htmlescape($this->fields['uuid']) . "</td>";
 
         echo "</tr>";
         echo "<tr class='tab_bg_1'>";
 
         $rule = new RuleImportAsset();
         $rule->getFromDB($this->fields['rules_id']);
-        echo "<th>" .  Rule::getTypeName(1) . "</th>";
+        echo "<th>" . htmlescape(Rule::getTypeName(1)) . "</th>";
         echo "<td>";
         echo $rule->getLink();
 
@@ -219,17 +221,17 @@ class RefusedEquipment extends CommonDBTM
 
         $entity = new Entity();
         $entity->getFromDB($this->fields['entities_id']);
-        echo "<th>" .  Entity::getTypeName(1) . "</th>";
+        echo "<th>" . htmlescape(Entity::getTypeName(1)) . "</th>";
         echo "<td>" . $entity->getLink() . "</td>";
 
         echo "</tr>";
         echo "<tr class='tab_bg_1'>";
 
-        echo "<th>" .  IPAddress::getTypeName(1) . "</th>";
-        echo "<td>" . implode(', ', importArrayFromDB($this->fields['ip'])) . "</td>";
+        echo "<th>" . htmlescape(IPAddress::getTypeName(1)) . "</th>";
+        echo "<td>" . htmlescape(implode(', ', importArrayFromDB($this->fields['ip']))) . "</td>";
 
-        echo "<th>" .  __('MAC address') . "</th>";
-        echo "<td>" . implode(', ', importArrayFromDB($this->fields['mac'])) . "</td>";
+        echo "<th>" . __s('MAC address') . "</th>";
+        echo "<td>" . htmlescape(implode(', ', importArrayFromDB($this->fields['mac']))) . "</td>";
 
         echo "</tr>";
         $this->showInventoryInfo();
@@ -244,7 +246,7 @@ class RefusedEquipment extends CommonDBTM
         return true;
     }
 
-    public static function canPurge()
+    public static function canPurge(): bool
     {
         return static::canUpdate();
     }
@@ -258,26 +260,26 @@ class RefusedEquipment extends CommonDBTM
     {
         $status = $request->getInventoryStatus();
 
-        if ($status['itemtype'] === RefusedEquipment::class) {
+        if ($status['itemtype'] === self::class) {
             Session::addMessageAfterRedirect(
-                __('Inventory is still refused.')
+                __s('Inventory is still refused.')
             );
-            return $this->getSearchURL();
-        } else {
-            $this->delete(['id' => $this->fields['id']], true);
-            Session::addMessageAfterRedirect(
-                __('Inventory is successful, refused entry log has been removed.')
-            );
-
-            $item = new $status['itemtype']();
-            if (isset($status['items_id'])) {
-                $item->getFromDB($status['items_id']);
-                $redirect_url = $item->getLinkURL();
-            } else {
-                $redirect_url = $item->getSearchURL();
-            }
-
-            return $redirect_url;
+            return static::getSearchURL();
         }
+
+        $this->delete(['id' => $this->fields['id']], true);
+        Session::addMessageAfterRedirect(
+            __s('Inventory is successful, refused entry log has been removed.')
+        );
+
+        $item = getItemForItemtype($status['itemtype']);
+        if (isset($status['items_id'])) {
+            $item->getFromDB($status['items_id']);
+            $redirect_url = $item->getLinkURL();
+        } else {
+            $redirect_url = $item->getSearchURL();
+        }
+
+        return $redirect_url;
     }
 }
