@@ -33,26 +33,33 @@
  * ---------------------------------------------------------------------
  */
 
+use Glpi\Application\View\TemplateRenderer;
+use Glpi\DBAL\QueryExpression;
+use Glpi\DBAL\QuerySubQuery;
+
+use function Safe\preg_match_all;
+use function Safe\preg_replace;
+
 class RuleAction extends CommonDBChild
 {
-   // From CommonDBChild
-    public static $itemtype        = "Rule";
+    // From CommonDBChild
+    /**
+     * @var class-string<Rule>
+     */
+    public static $itemtype        = Rule::class;
     public static $items_id        = 'rules_id';
     public $dohistory              = true;
     public $auto_message_on_action = false;
 
-    /**
-     * @since 0.84
-     **/
+
     public function getForbiddenStandardMassiveAction()
     {
-
         $forbidden   = parent::getForbiddenStandardMassiveAction();
         $forbidden[] = 'update';
 
-        if (isset($_POST['rule_class_name']) && is_subclass_of(\Rule::class, $_POST['rule_class_name'])) {
+        if (isset($_POST['rule_class_name']) && is_subclass_of(Rule::class, $_POST['rule_class_name'])) {
             $rule = getItemForItemtype($_POST['rule_class_name']);
-            if ($rule->maxActionsCount() == 1) {
+            if ($rule instanceof Rule && $rule->maxActionsCount() == 1) {
                 $forbidden[] = 'clone';
             }
         }
@@ -60,25 +67,18 @@ class RuleAction extends CommonDBChild
         return $forbidden;
     }
 
-
     /**
-     * @param string $rule_type
+     * @param class-string<Rule> $rule_type
      **/
     public function __construct($rule_type = 'Rule')
     {
         static::$itemtype = $rule_type;
     }
 
-
-    /**
-     * @since 0.84.3
-     *
-     * @see CommonDBTM::post_getFromDB()
-     */
     public function post_getFromDB()
     {
 
-       // Get correct itemtype if defult one is used
+        // Get correct itemtype if defult one is used
         if (static::$itemtype == 'Rule') {
             $rule = new Rule();
             if ($rule->getFromDB($this->fields['rules_id'])) {
@@ -87,22 +87,18 @@ class RuleAction extends CommonDBChild
         }
     }
 
-
-    /**
-     * Get title used in rule
-     *
-     * @param integer $nb (default 0)
-     *
-     * @return string Title of the rule
-     **/
     public static function getTypeName($nb = 0)
     {
         return _n('Action', 'Actions', $nb);
     }
 
+    public static function getIcon()
+    {
+        return "ti ti-player-play";
+    }
+
     protected function computeFriendlyName()
     {
-
         if ($rule = getItemForItemtype(static::$itemtype)) {
             $action_row = $rule->getMinimalActionText($this->fields);
             $action_text = trim(preg_replace(['/<td[^>]*>/', '/<\/td>/'], [' ', ''], $action_row));
@@ -111,86 +107,67 @@ class RuleAction extends CommonDBChild
         return '';
     }
 
-    /**
-     * @since 0.84
-     *
-     * @see CommonDBChild::post_addItem()
-     **/
     public function post_addItem()
     {
-
         parent::post_addItem();
         if (
             isset($this->input['rules_id'])
             && ($realrule = Rule::getRuleObjectByID($this->input['rules_id']))
         ) {
             $realrule->update(['id'       => $this->input['rules_id'],
-                'date_mod' => $_SESSION['glpi_currenttime']
+                'date_mod' => $_SESSION['glpi_currenttime'],
             ]);
         }
     }
 
-
-    /**
-     * @since 0.84
-     *
-     * @see CommonDBTM::post_purgeItem()
-     **/
     public function post_purgeItem()
     {
-
         parent::post_purgeItem();
         if (
             isset($this->fields['rules_id'])
             && ($realrule = Rule::getRuleObjectByID($this->fields['rules_id']))
         ) {
             $realrule->update(['id'       => $this->fields['rules_id'],
-                'date_mod' => $_SESSION['glpi_currenttime']
+                'date_mod' => $_SESSION['glpi_currenttime'],
             ]);
         }
     }
 
-
-    /**
-     * @since 0.84
-     **/
     public function prepareInputForAdd($input)
     {
-
-        if (!isset($input['field']) || empty($input['field'])) {
+        if (empty($input['field'])) {
             return false;
         }
         return parent::prepareInputForAdd($input);
     }
 
-
     public function rawSearchOptions()
     {
         $tab = [];
 
         $tab[] = [
             'id'                 => '1',
-            'table'              => $this->getTable(),
+            'table'              => static::getTable(),
             'field'              => 'action_type',
             'name'               => self::getTypeName(1),
             'massiveaction'      => false,
             'datatype'           => 'specific',
-            'additionalfields'   => ['rules_id']
+            'additionalfields'   => ['rules_id'],
         ];
 
         $tab[] = [
             'id'                 => '2',
-            'table'              => $this->getTable(),
+            'table'              => static::getTable(),
             'field'              => 'field',
             'name'               => _n('Field', 'Fields', Session::getPluralNumber()),
             'massiveaction'      => false,
             'datatype'           => 'specific',
-            'additionalfields'   => ['rules_id']
+            'additionalfields'   => ['rules_id'],
         ];
 
         $tab[] = [
             'id'                 => '3',
-            'table'              => $this->getTable(),
+            'table'              => static::getTable(),
             'field'              => 'value',
             'name'               => __('Value'),
             'massiveaction'      => false,
@@ -201,14 +178,6 @@ class RuleAction extends CommonDBChild
         return $tab;
     }
 
-
-    /**
-     * @since 0.84
-     *
-     * @param $field
-     * @param $values
-     * @param $options   array
-     **/
     public static function getSpecificValueToDisplay($field, $values, array $options = [])
     {
 
@@ -223,14 +192,15 @@ class RuleAction extends CommonDBChild
                     && !empty($values['rules_id'])
                     && $generic_rule->getFromDB($values['rules_id'])
                 ) {
-                    if ($rule = getItemForItemtype($generic_rule->fields["sub_type"])) {
-                        return $rule->getAction($values[$field]);
+                    $rule = getItemForItemtype($generic_rule->fields["sub_type"]);
+                    if ($rule instanceof Rule) {
+                        return htmlescape($rule->getActionName($values[$field]));
                     }
                 }
                 break;
 
             case 'action_type':
-                return self::getActionByID($values[$field]);
+                return htmlescape(self::getActionByID($values[$field]));
 
             case 'value':
                 if (!isset($values["field"]) || !isset($values["action_type"])) {
@@ -242,12 +212,13 @@ class RuleAction extends CommonDBChild
                     && !empty($values['rules_id'])
                     && $generic_rule->getFromDB($values['rules_id'])
                 ) {
-                    if ($rule = getItemForItemtype($generic_rule->fields["sub_type"])) {
-                        return $rule->getCriteriaDisplayPattern(
+                    $rule = getItemForItemtype($generic_rule->fields["sub_type"]);
+                    if ($rule instanceof Rule) {
+                        return htmlescape($rule->getCriteriaDisplayPattern(
                             $values["criteria"],
                             $values["condition"],
                             $values[$field]
-                        );
+                        ));
                     }
                 }
                 break;
@@ -255,15 +226,6 @@ class RuleAction extends CommonDBChild
         return parent::getSpecificValueToDisplay($field, $values, $options);
     }
 
-
-    /**
-     * @since 0.84
-     *
-     * @param $field
-     * @param $name               (default '')
-     * @param $values             (default '')
-     * @param $options      array
-     **/
     public static function getSpecificValueToSelect($field, $name = '', $values = '', array $options = [])
     {
         if (!is_array($values)) {
@@ -278,7 +240,8 @@ class RuleAction extends CommonDBChild
                     && !empty($values['rules_id'])
                     && $generic_rule->getFromDB($values['rules_id'])
                 ) {
-                    if ($rule = getItemForItemtype($generic_rule->fields["sub_type"])) {
+                    $rule = getItemForItemtype($generic_rule->fields["sub_type"]);
+                    if ($rule instanceof Rule) {
                         $options['value'] = $values[$field];
                         $options['name']  = $name;
                         return $rule->dropdownActions($options);
@@ -297,7 +260,7 @@ class RuleAction extends CommonDBChild
                         'name'        => $name,
                         'value'       => $values[$field],
                         'alreadyused' => false,
-                        'display'     => false
+                        'display'     => false,
                     ]);
                 }
                 break;
@@ -312,9 +275,10 @@ class RuleAction extends CommonDBChild
                     && !empty($values['rules_id'])
                     && $generic_rule->getFromDB($values['rules_id'])
                 ) {
-                    if ($rule = getItemForItemtype($generic_rule->fields["sub_type"])) {
-                       /// TODO review it : need to pass display param and others...
-                        return $rule->displayActionSelectPattern($values);
+                    $rule = getItemForItemtype($generic_rule->fields["sub_type"]);
+                    if ($rule instanceof Rule) {
+                        /// TODO review it : need to pass display param and others...
+                        return (new static())->displayActionSelectPattern($values);
                     }
                 }
                 break;
@@ -322,23 +286,21 @@ class RuleAction extends CommonDBChild
         return parent::getSpecificValueToSelect($field, $name, $values, $options);
     }
 
-
     /**
      * Get all actions for a given rule
      *
-     * @param $ID the rule_description ID
+     * @param integer $ID the rule_description ID
      *
      * @return array of RuleAction objects
      **/
     public function getRuleActions($ID)
     {
-        /** @var \DBmysql $DB */
         global $DB;
 
         $iterator = $DB->request([
-            'FROM'   => $this->getTable(),
+            'FROM'   => static::getTable(),
             'WHERE'  => [static::$items_id => $ID],
-            'ORDER'  => 'id'
+            'ORDER'  => 'id',
         ]);
 
         $rules_actions = [];
@@ -350,18 +312,16 @@ class RuleAction extends CommonDBChild
         return $rules_actions;
     }
 
-
     /**
      * Add an action
      *
-     * @param $action    action type
-     * @param $ruleid    rule ID
-     * @param $field     field name
-     * @param $value     value
+     * @param string $action Action type
+     * @param integer $ruleid Rule ID
+     * @param string $field Field name
+     * @param mixed $value Value
      **/
     public function addActionByAttributes($action, $ruleid, $field, $value)
     {
-
         $input = [
             'action_type'     => $action,
             'field'           => $field,
@@ -371,40 +331,35 @@ class RuleAction extends CommonDBChild
         $this->add($input);
     }
 
-
     /**
      * Display a dropdown with all the possible actions
      *
-     * @param $options   array of possible options:
-     *    - subtype
-     *    - name
-     *    - field
-     *    - value
-     *    - alreadyused
-     *    - display
+     * @param array{subtype: class-string<Rule>, name: string, field?: string, value?: string, alreadyused: bool, display?: bool} $options
+     * <ul>
+     *     <li>subtype: the itemtype of the rule</li>
+     *     <li>name: the name of the dropdown</li>
+     *     <li>field: the field name</li>
+     *     <li>value: the value</li>
+     *     <li>alreadyused: if an action of the same type was already used for the rule (default false)</li>
+     *     <li>display: if the dropdown should be displayed< (default true)/li>
+     * </ul>
+     * @return string|int|false Returns the dropdown HTML if display is false, otherwise the random number used to create the dropdown is returned.
      **/
-    public static function dropdownActions($options = [])
+    public static function dropdownActions($options)
     {
-
-        $p = [
+        $p = array_replace([
             'subtype'     => '',
             'name'        => '',
             'field'       => '',
             'value'       => '',
             'alreadyused' => false,
             'display'     => true,
-        ];
-
-        if (is_array($options) && count($options)) {
-            foreach ($options as $key => $val) {
-                $p[$key] = $val;
-            }
-        }
+        ], $options);
 
         if ($rule = getItemForItemtype($p['subtype'])) {
             $actions_options = $rule->getAllActions();
             $actions         = ["assign"];
-           // Manage permit several.
+            // Manage permit several.
             $field = $p['field'];
             if ($p['alreadyused']) {
                 if (!isset($actions_options[$field]['permitseveral'])) {
@@ -423,16 +378,19 @@ class RuleAction extends CommonDBChild
             }
 
             return Dropdown::showFromArray($p['name'], $elements, ['value'   => $p['value'],
-                'display' => $p['display']
+                'display' => $p['display'],
             ]);
         }
+        return '';
     }
 
-
+    /**
+     * @return array<string, string>
+     */
     public static function getActions()
     {
-
-        return ['assign'              => __('Assign'),
+        return [
+            'assign'              => __('Assign'),
             'append'              => __('Add'),
             'regex_result'        => __('Assign the value from regular expression'),
             'append_regex_result' => __('Add the result of regular expression'),
@@ -447,32 +405,27 @@ class RuleAction extends CommonDBChild
             'fromuser'            => __('Copy from user'),
             'defaultfromuser'     => __('Copy default from user'),
             'firstgroupfromuser'  => __('Copy first group from user'),
-            'fromitem'            => __('Copy from item')
+            'fromitem'            => __('Copy from item'),
         ];
     }
 
-
     /**
-     * @param $ID
-     **/
+     * @param string $ID
+     * @return string
+     */
     public static function getActionByID($ID)
     {
-
         $actions = self::getActions();
-        if (isset($actions[$ID])) {
-            return $actions[$ID];
-        }
-        return '';
+        return $actions[$ID] ?? '';
     }
 
-
     /**
-     * @param $action
-     * @param $regex_result
+     * @param string $action
+     * @param array $regex_result
+     * @return string
      **/
     public static function getRegexResultById($action, $regex_result)
     {
-
         $results = [];
 
         if (count($regex_result) > 0) {
@@ -480,7 +433,7 @@ class RuleAction extends CommonDBChild
                 foreach ($results[1] as $result) {
                     $action = str_replace(
                         "#$result",
-                        (isset($regex_result[$result]) ? $regex_result[$result] : ''),
+                        $regex_result[$result] ?? '',
                         $action
                     );
                 }
@@ -489,14 +442,12 @@ class RuleAction extends CommonDBChild
         return $action;
     }
 
-
     /**
-     * @param $rules_id
-     * @param $sub_type
+     * @param integer $rules_id
+     * @param class-string<Rule> $sub_type
      **/
     public function getAlreadyUsedForRuleID($rules_id, $sub_type)
     {
-        /** @var \DBmysql $DB */
         global $DB;
 
         if ($rule = getItemForItemtype($sub_type)) {
@@ -505,7 +456,7 @@ class RuleAction extends CommonDBChild
             $actions = [];
             $iterator = $DB->request([
                 'SELECT' => 'field',
-                'FROM'   => $this->getTable(),
+                'FROM'   => static::getTable(),
                 'WHERE'  => [static::$items_id => $rules_id],
             ]);
 
@@ -523,9 +474,8 @@ class RuleAction extends CommonDBChild
         }
     }
 
-
     /**
-     * @param $options   array
+     * @param array $options
      **/
     public function displayActionSelectPattern($options = [])
     {
@@ -557,6 +507,10 @@ class RuleAction extends CommonDBChild
                 $actions = Rule::getActionsByType($options["sub_type"]);
                 if (isset($actions[$options["field"]]['type'])) {
                     switch ($actions[$options["field"]]['type']) {
+                        case "dropdown_entity":
+                            $param['toadd'] = [-1 => __('Full structure')];
+                            // Intentional fall-through to handle dropdown cases
+                            // no break
                         case "dropdown":
                             $table   = $actions[$options["field"]]['table'];
                             $param['name'] = "value";
@@ -582,6 +536,12 @@ class RuleAction extends CommonDBChild
                         case "dropdown_users":
                             $param['name']  = 'value';
                             $param['right'] = 'all';
+                            $param['toadd'] = [
+                                [
+                                    'id'    => 'requester_manager',
+                                    'text'  => __("Requester's manager"),
+                                ],
+                            ];
                             User::dropdown($param);
                             $display = true;
                             break;
@@ -593,9 +553,9 @@ class RuleAction extends CommonDBChild
                             break;
 
                         case "dropdown_impact":
-                              $param['name']  = 'value';
-                              Ticket::dropdownImpact($param);
-                              $display = true;
+                            $param['name']  = 'value';
+                            Ticket::dropdownImpact($param);
+                            $display = true;
                             break;
 
                         case "dropdown_priority":
@@ -608,7 +568,12 @@ class RuleAction extends CommonDBChild
 
                         case "dropdown_status":
                             $param['name']  = 'value';
-                            Ticket::dropdownStatus($param);
+                            if (is_a($_POST['sub_type'], RuleCommonITILObject::class, true)) {
+                                $itil = $_POST['sub_type']::getItemtype();
+                                $itil::dropdownStatus($param);
+                            } else {
+                                Ticket::dropdownStatus($param);
+                            }
                             $display = true;
                             break;
 
@@ -632,18 +597,19 @@ class RuleAction extends CommonDBChild
 
                         case "dropdown_users_validate":
                             $used = [];
-                            if ($item = getItemForItemtype($options["sub_type"])) {
+                            $item = getItemForItemtype($options["sub_type"]);
+                            if ($item instanceof Rule) {
                                 $rule_data = getAllDataFromTable(
                                     self::getTable(),
                                     [
                                         'action_type'           => 'add_validation',
                                         'field'                 => 'users_id_validate',
-                                        $item->getRuleIdField() => $options[$item->getRuleIdField()]
+                                        $item->getRuleIdField() => $options[$item->getRuleIdField()],
                                     ]
                                 );
 
                                 foreach ($rule_data as $data) {
-                                       $used[] = $data['value'];
+                                    $used[] = $data['value'];
                                 }
                             }
                             $param['name']  = 'value';
@@ -655,17 +621,18 @@ class RuleAction extends CommonDBChild
 
                         case "dropdown_groups_validate":
                             $used = [];
-                            if ($item = getItemForItemtype($options["sub_type"])) {
+                            $item = getItemForItemtype($options["sub_type"]);
+                            if ($item instanceof Rule) {
                                 $rule_data = getAllDataFromTable(
                                     self::getTable(),
                                     [
                                         'action_type'           => 'add_validation',
                                         'field'                 => 'groups_id_validate',
-                                        $item->getRuleIdField() => $options[$item->getRuleIdField()]
+                                        $item->getRuleIdField() => $options[$item->getRuleIdField()],
                                     ]
                                 );
                                 foreach ($rule_data as $data) {
-                                     $used[] = $data['value'];
+                                    $used[] = $data['value'];
                                 }
                             }
 
@@ -673,8 +640,8 @@ class RuleAction extends CommonDBChild
                             $param['condition'] = [new QuerySubQuery([
                                 'SELECT' => ['COUNT' => ['users_id']],
                                 'FROM'   => 'glpi_groups_users',
-                                'WHERE'  => ['groups_id' => new \QueryExpression('glpi_groups.id')]
-                            ])
+                                'WHERE'  => ['groups_id' => new QueryExpression('glpi_groups.id')],
+                            ]),
                             ];
                             $param['right']     = ['validate_incident', 'validate_request'];
                             $param['used']      = $used;
@@ -682,22 +649,21 @@ class RuleAction extends CommonDBChild
                             $display            = true;
                             break;
 
-                        case "dropdown_validation_percent":
-                            $ticket = new Ticket();
-                            echo $ticket->getValueToSelect('validation_percent', 'value', $param['value']);
-                            $display       = true;
-                            break;
-
-                        case "dropdown_validation_status":
-                            TicketValidation::dropdownStatus('value', [
-                                'global' => true,
-                                'value' => $param['value'],
-                            ]);
+                        case "percent":
+                            Dropdown::showNumber(
+                                'value',
+                                [
+                                    'value' => $param['value'],
+                                    'min'   => 0,
+                                    'max'   => 100,
+                                ]
+                            );
                             $display = true;
                             break;
 
                         default:
-                            if ($rule = getItemForItemtype($options["sub_type"])) {
+                            $rule = getItemForItemtype($options["sub_type"]);
+                            if ($rule instanceof Rule) {
                                 $display = $rule->displayAdditionalRuleAction($actions[$options["field"]], $param['value']);
                             }
                             break;
@@ -710,81 +676,32 @@ class RuleAction extends CommonDBChild
         }
     }
 
-    /** form for rule action
-     *
-     * @since 0.85
-     *
-     * @param $ID      integer : Id of the action
-     * @param $options array of possible options:
-     *     - rule Object : the rule
-     **/
     public function showForm($ID, array $options = [])
     {
-        /** @var array $CFG_GLPI */
-        global $CFG_GLPI;
-
-       // Yllen: you always have parent for action
+        // Yllen: you always have parent for action
         $rule = $options['parent'];
 
-        if ($ID > 0) {
+        if (!static::isNewID($ID)) {
             $this->check($ID, READ);
         } else {
-           // Create item
+            // Create item
             $options[static::$items_id] = $rule->getField('id');
-
-           //force itemtype of parent
+            // force itemtype of parent
             static::$itemtype = get_class($rule);
-
             $this->check(-1, CREATE, $options);
         }
-        $this->showFormHeader($options);
 
-        echo "<tr class='tab_bg_1 center'>";
-        echo "<td>" . _n('Action', 'Actions', 1) . "</td><td colspan='3'>";
-        echo "<input type='hidden' name='" . $rule->getRuleIdField() . "' value='" .
-             $this->fields[static::$items_id] . "'>";
-        $used = $this->getAlreadyUsedForRuleID($this->fields[static::$items_id], $rule->getType());
-       // On edit : unset selected value
-        if (
-            $ID
-            && isset($used[$this->fields['field']])
-        ) {
+        $used = $this->getAlreadyUsedForRuleID($rule->getID(), get_class($rule));
+        if (isset($used[$this->fields['field']]) && !static::isNewID($ID)) {
             unset($used[$this->fields['field']]);
         }
-        $rand   = $rule->dropdownActions(['value' => $this->fields['field'],
-            'used'  => $used
+        TemplateRenderer::getInstance()->display('pages/admin/rules/action.html.twig', [
+            'rule' => $rule,
+            'rules_id_field' => static::$items_id,
+            'item' => $this,
+            'used_actions' => $used,
+            'rand' => mt_rand(),
         ]);
-        $params = ['field'                 => '__VALUE__',
-            'sub_type'              => $rule->getType(),
-            'ruleactions_id'        => $this->getID(),
-            $rule->getRuleIdField() => $this->fields[static::$items_id]
-        ];
-
-        Ajax::updateItemOnSelectEvent(
-            "dropdown_field$rand",
-            "action_span",
-            $CFG_GLPI["root_doc"] . "/ajax/ruleaction.php",
-            $params
-        );
-
-        if (isset($this->fields['field']) && !empty($this->fields['field'])) {
-            $params['field']       = $this->fields['field'];
-            $params['action_type'] = $this->fields['action_type'];
-            $params['value']       = $this->fields['value'];
-            echo "<script type='text/javascript' >\n";
-            echo "$(function() {";
-            Ajax::updateItemJsCode(
-                "action_span",
-                $CFG_GLPI["root_doc"] . "/ajax/ruleaction.php",
-                $params
-            );
-            echo '});</script>';
-        }
-        echo "</td></tr>";
-        echo "<tr><td colspan='4'><span id='action_span'>\n";
-        echo "</span></td>\n";
-        echo "</tr>\n";
-        $this->showFormButtons($options);
 
         return true;
     }
