@@ -35,8 +35,5 @@
 
 class RuleDictionnaryDropdown extends Rule
 {
-   // From Rule
-    public $can_sort      = true;
-
     public static $rightname     = 'rule_dictionnary_dropdown';
 }
